@@ -1,10 +1,10 @@
-#Electro-social routines for R
+#Ohmic/Electro-social routines for R
 #How we get importance of ties, and to a point individuals,
 #to information communication.
 #Andrew C. Thomas
-#May 4, 2009
+#Last updated: January 20, 2010
 
-debug.mode <- 0
+debug.mode <- 1
 
 #produces all ordered pairs between 1 and nn.
 pair.sequence <- function(nn) {
@@ -18,15 +18,26 @@ pair.sequence <- function(nn) {
 }
 
 #inputs an edgelist, a valued edgelist or a valued dyad list.
-make.sociomatrix.from.edges <- function(inputmat,symmetric=FALSE) {
+make.sociomatrix.from.edges <- function(inputmat, symmetric=FALSE, fidelities=NULL) {
 
   if (dim(inputmat)[2]>4) stop("Incorrect dimensionality for edge matrix.")
+  fidelity.dim <- 2-1*(dim(inputmat)[2]<4)
+  if (is.null(fidelities)) {
+    fidelities <- array(1,c(dim(inputmat)[1],fidelity.dim))
+  } else {
+    fidelities <- cbind(fidelities)
+    if (dim(fidelities)[1]!=dim(inputmat)[1]) stop(paste("Number of rows in the fidelity term,",dim(fidelities)[1]," does not match the rows in the tie matrix,", dim(inputmat)[1], "."))
+    #if (fidelity.dim != dim(fidelities)[2]) stop(paste("The number of columns in the fidelity term, does not match the tie matrix.")
+    if (fidelity.dim == 2 & dim(fidelities)[2] == 1) fidelities <- cbind(fidelities, fidelities)
+    if (fidelity.dim == 1 & dim(fidelities)[2] == 2) stop(paste("There are two columns in the fidelity term when only one is possible."))
+  }
   
   inputmat <- as.matrix(inputmat)
   id.names <- sort(unique(as.vector(inputmat[,1:2])))
   rows <- length(id.names)
 
   outmat <- array(0,rep(rows,2))
+  out.fidelity <- array(1,rep(rows,2))
   cols <- dim(inputmat)[2]
 
   #relabel.
@@ -37,34 +48,49 @@ make.sociomatrix.from.edges <- function(inputmat,symmetric=FALSE) {
   inputmat <- array(as.numeric(inputmat),dim(inputmat))
   
   if (cols==2 & symmetric) for (kk in 1:dim(inputmat)[1]) {
-    outmat[inputmat[kk,1],inputmat[kk,2]] <- outmat[inputmat[kk,2],inputmat[kk,1]] <- 1 }
+    outmat[inputmat[kk,1],inputmat[kk,2]] <- outmat[inputmat[kk,2],inputmat[kk,1]] <- 1
+    out.fidelity[inputmat[kk,1],inputmat[kk,2]] <- out.fidelity[inputmat[kk,2],inputmat[kk,1]] <- fidelities[kk,1]
+  }
   if (cols==2 & !symmetric) for (kk in 1:dim(inputmat)[1]) {
-    outmat[inputmat[kk,1],inputmat[kk,2]] <- 1 }
+    outmat[inputmat[kk,1],inputmat[kk,2]] <- 1
+    out.fidelity[inputmat[kk,1],inputmat[kk,2]] <- fidelities[kk,1]
+  }
 
   if (cols==3 & symmetric) for (kk in 1:dim(inputmat)[1]) {
-    outmat[inputmat[kk,1],inputmat[kk,2]] <- outmat[inputmat[kk,2],inputmat[kk,1]] <- inputmat[kk,3] }
+    outmat[inputmat[kk,1],inputmat[kk,2]] <- outmat[inputmat[kk,2],inputmat[kk,1]] <- inputmat[kk,3]
+    out.fidelity[inputmat[kk,1],inputmat[kk,2]] <- out.fidelity[inputmat[kk,2],inputmat[kk,1]] <- fidelities[kk,1]
+  }
   if (cols==3 & !symmetric) for (kk in 1:dim(inputmat)[1]) {
-    outmat[inputmat[kk,1],inputmat[kk,2]] <- inputmat[kk,3] }
+    outmat[inputmat[kk,1],inputmat[kk,2]] <- inputmat[kk,3]
+    out.fidelity[inputmat[kk,1],inputmat[kk,2]] <- fidelities[kk,1]
+  }
   
   if (cols==4) for (kk in 1:dim(inputmat)[1]) {
     outmat[inputmat[kk,1],inputmat[kk,2]] <- inputmat[kk,3]
     outmat[inputmat[kk,2],inputmat[kk,1]] <- inputmat[kk,4]
+    out.fidelity[inputmat[kk,1],inputmat[kk,2]] <- fidelities[kk,1]
+    out.fidelity[inputmat[kk,2],inputmat[kk,1]] <- fidelities[kk,2]
   }
   
   rownames(outmat) <- colnames(outmat) <- id.names
-  return(outmat)
+  rownames(out.fidelity) <- colnames(out.fidelity) <- id.names
+
+  output <- list(sociomatrix=outmat, fidelities=out.fidelity)
+  return(output)
 }
 
 
 #create a sociomatrix based on distance-1 adjacency in a lattice.
 make.sociomatrix.from.lattice <- function(pts.nby2) {
   n.pts <- dim(pts.nby2)[1]
-  return(sapply(1:n.pts,FUN=function(ii)
-                sapply(1:n.pts,FUN=function(kk)
-                       1*(sum((pts.nby2[ii,]-pts.nby2[kk,])^2)==1)
-                       )
-                )
-         )
+  sociomatrix <- sapply(1:n.pts,FUN=function(ii)
+                        sapply(1:n.pts,FUN=function(kk)
+                               1*(sum((pts.nby2[ii,]-pts.nby2[kk,])^2)==1)
+                               )
+                        )
+  fidelities <- array(1,dim(sociomatrix))
+  return(list(sociomatrix=sociomatrix, fidelities=fidelities))
+  
 }
 
 #now in C! It's actually Floyd-Warshall... with absolute values.
@@ -99,10 +125,45 @@ geodesic.mat <- function(sociomatrix) {
 }
 
 
-#assembles weakly connected components.
 network.components <- function(sociomatrix,
+                                    minimum.relative.strength.for.tie=1e-8) {
+
+  n.pts <- dim(sociomatrix)[1]
+  maxtie <- max(sociomatrix)
+  sociomatrix[sociomatrix < minimum.relative.strength.for.tie*maxtie] <- 0
+
+  #cascade.
+  components <- 1:n.pts
+
+  dothis <- .C("network_components",
+               sociomatrix=as.double(sociomatrix),
+               nn = as.integer(n.pts),
+               comps = as.integer(components))
+  outputcomps <- dothis$comps
+
+  comps <- sort(unique(outputcomps))
+  components <- list(NA)
+  component.vector <- rep(NA, n.pts)
+
+  for (cc in 1:length(comps)) {
+    component.vector[outputcomps==comps[cc]] <- cc
+    components[[cc]] <- which(outputcomps==comps[cc])
+  }
+
+  component.vector <- cbind(component.vector)
+  rownames(component.vector) <- rownames(sociomatrix)
+  
+  out <- list(components=components,
+              component.vector=component.vector)
+  return(out)
+}
+
+
+#assembles weakly connected components.
+network.components.old <- function(sociomatrix,
                                pseudo.diameter.bridge=2,
                                minimum.relative.strength.for.tie=1e-8) {
+  
   n.pts <- dim(sociomatrix)[1]
   maxtie <- max(sociomatrix)
   sociomatrix[sociomatrix < minimum.relative.strength.for.tie*maxtie] <- 0
@@ -139,8 +200,13 @@ network.components <- function(sociomatrix,
   return(out)
 }
 
-electrograph <- function(input, symmetric=TRUE, perform.exam=TRUE,
-                         enemies.allowed=FALSE, ...) {
+
+electrograph <- function(input, symmetric=TRUE,
+                         solve.for.shortest.paths=TRUE,
+                         ohmic.properties=TRUE,
+                         fidelities=NULL, verbose=FALSE,
+                         substitute.names=NULL,...) {
+
   input <- as.matrix(input)
   
   if (dim(input)[1] == dim(input)[2] & is.numeric(input)) {
@@ -148,41 +214,80 @@ electrograph <- function(input, symmetric=TRUE, perform.exam=TRUE,
     if (all(is.null(rownames(sociomatrix)))) {
       rownames(sociomatrix) <- colnames(sociomatrix) <- 1:(dim(sociomatrix)[1])
     }
+    if (!is.null(fidelities)) {
+      if (!all(dim(fidelities)==dim(sociomatrix))) stop ("Dimensions of the fidelity matrix do not match the sociomatrix.")
+    }
   } else {
-    sociomatrix <- make.sociomatrix.from.edges(input,symmetric)
+    sociomatrix.maker <- make.sociomatrix.from.edges(input, symmetric, fidelities)
+    sociomatrix <- sociomatrix.maker$sociomatrix
+    fidelities <- sociomatrix.maker$fidelities
   }
 
+  if (is.null(fidelities)) fidelities <- array(1,dim(sociomatrix))
+  if (max(abs(fidelities))>1) stop("At least one fidelity value is outside the range (-1,1).")
   
   #reset symmetric now.
   symmetric <- all(sociomatrix==t(sociomatrix))
   nonnegative <- all(sociomatrix >= 0)
-  if (!nonnegative & !enemies.allowed) stop ("input contains negative tie strengths, which are disallowed if enemies.allowed is not selected.")
+  if (!nonnegative) {
+    message ("electrograph says: input contains negative tie strengths. Interpreting these as antagonistic connections with fidelity -1.") #To include \"enemies\" ties, use the \"fidelities\" option.")
+    fidelities[sociomatrix<0] <- -1
+    sociomatrix <- abs(sociomatrix)
+  }
+
+  #do we have a vector of names to substitute?
+  if (!is.null(substitute.names)) {
+    substitute.names <- as.matrix(substitute.names)
+    if (dim(substitute.names)[2]!=2) stop (paste("The substituted node name matrix should be k-by-2; it is reading as dimension",dim(substitute.names)[2],"."))
+
+    sub.rows <- match(substitute.names[,1],rownames(sociomatrix))
+    rownames(sociomatrix)[sub.rows] <- colnames(sociomatrix)[sub.rows] <- substitute.names[,2]
+    rownames(fidelities)[sub.rows] <- colnames(fidelities)[sub.rows] <- substitute.names[,2]
+  }
+
 
-  #if (!nonnegative & enemies.allowed) {
-  #  message("For the time being, `enemies' mode is disabled. Setting entries to be nonnegative.")
-  #  sociomatrix <- abs(sociomatrix)}
   
-  pieces <- network.components(sociomatrix, pseudo.diameter.bridge=2)
+  pieces <- network.components(sociomatrix)
   sociomatrices <- list(NA)
-
+  fidelity.pieces <- list(NA)
+  
   for (kk in 1:length(pieces$components)) {
     sociomatrices[[kk]] <- array(sociomatrix[pieces$components[[kk]],pieces$components[[kk]]],
                                  rep(length(pieces$components[[kk]]),2))
     rownames(sociomatrices[[kk]]) <- colnames(sociomatrices[[kk]]) <-
-      rownames(sociomatrix)[pieces$components[[kk]]]               
-  }
+      rownames(sociomatrix)[pieces$components[[kk]]]
 
-  out <- list(grand.sociomatrix=sociomatrix, sociomatrices=sociomatrices,
+    fidelity.pieces[[kk]] <- array(fidelities[pieces$components[[kk]],pieces$components[[kk]]],
+                                 rep(length(pieces$components[[kk]]),2))
+    rownames(fidelity.pieces[[kk]]) <- colnames(fidelity.pieces[[kk]]) <-
+      rownames(sociomatrix)[pieces$components[[kk]]]
+
+    if (debugmode) print(fidelity.pieces[[kk]])
+  }
+  
+  out <- list(grand.sociomatrix=sociomatrix,
+              sociomatrices=sociomatrices,
+              grand.fidelity=fidelities,
+              fidelities=fidelity.pieces,
+              components=pieces$components,
               component.vector=pieces$component.vector,
-              geodesic=pieces$geodesic, diameters=pieces$diameters,
-              global.pseudo.diameter=pieces$global.pseudo.diameter,
               symmetric=symmetric)
+
+  #geodesic=pieces$geodesic, diameters=pieces$diameters,
+  #global.pseudo.diameter=pieces$global.pseudo.diameter,
+              
   class(out) <- "electrograph"
 
-  if (debug.mode) message("Finished electrograph step 1.")
-  if (perform.exam) {
+  if (verbose) message("Finished electrograph object loading.")
+
+  if (solve.for.shortest.paths) {
+    out$geodesic <- geodesic.mat(sociomatrix)
+    if (verbose) message("Finished geodesic path length calculation.")
+  }
+  
+  if (ohmic.properties) {
     out <- electrograph.exam(out, ...)
-    if (debug.mode) message("Finished electrograph exam.")
+    if (debug.mode) message("Finished Ohmic properties.")
   }
   
   return (out)
@@ -209,17 +314,10 @@ summary.electrograph <- function(object, ...) {
 
   #writeLines ("Wake up, Neo.")
   
+  nn <- dim(object$grand.sociomatrix)[1]
+
   out.deg <- apply(object$grand.sociomatrix,1,sum)
   in.deg <- apply(object$grand.sociomatrix,2,sum)
-  
-  hold.geo <- 1/object$geodesic
-  diag(hold.geo) <- 0
-  out <- NULL
-  sp.close <- apply(hold.geo,1,mean)
-  
-  out <- data.frame(out.deg, in.deg, sp.close)
-  nn <- dim(object$grand.sociomatrix)[1]
-  
   #construct clustering measures:
   clusts <- .C("clustering_statistics_c",
                socio=as.double(object$grand.sociomatrix),
@@ -229,21 +327,33 @@ summary.electrograph <- function(object, ...) {
   transits <- clusts$transitives
   cycles <- clusts$cycles
 
-  out <- data.frame(out,transits,cycles)
+  out <- data.frame(out.deg, in.deg, transits, cycles)
 
   
+#  out <- data.frame(out,transits,cycles)
+
+  if (!is.null(object$geodesic)) {
+    hold.geo <- 1/object$geodesic
+    diag(hold.geo) <- 0
+    geo.close.out <- apply(hold.geo,1,mean)
+    geo.close.in <- apply(hold.geo,2,mean)
+    out <- data.frame(out, geo.close.out, geo.close.in)
+  }
+  
+  
   if (!is.null(object$distance.mat)) {
     distance.hold <- 1/object$distance.mat
     diag(distance.hold) <- 0
-    eg.close <- apply(distance.hold,1,mean)
-    out <- data.frame(out,eg.close)
+    ohm.close.out <- apply(distance.hold,1,mean)
+    ohm.close.in <- apply(distance.hold,2,mean)
+    out <- data.frame(out, ohm.close.out, ohm.close.in)
   }
 
-  if (!is.null(object$blk.curr.a)) {
+  if (!is.null(object$avg.current.black.a)) {
 #    current.cent <- apply(object$currents.node,1,mean)
-    between.a <- apply(object$blk.curr.a,1,sum)
-    between.v <- apply(object$blk.curr.v,1,sum)
-    between.p <- apply(object$blk.curr.p,1,sum)
+    between.a <- apply(object$avg.current.black.a,1,sum)
+    between.v <- apply(object$avg.current.black.v,1,sum)
+    between.p <- apply(object$avg.current.black.p,1,sum)
     out <- data.frame(out, between.a, between.v, between.p)
   }
   
@@ -255,14 +365,15 @@ print.electrograph <- function(x, ...) {
 
   for (kk in 1:length(x$sociomatrices)) {
     writeLines(paste("head(sociomatrices[[",kk,"]]):"))
-    print(head(x$sociomatrices[[kk]]))
-  }  
+    dd <- min(dim(x$sociomatrices[[kk]]),6)
+    print(x$sociomatrices[[kk]][1:dd, 1:dd])
+  }
 
   writeLines(paste("component.vector"))
   print(t(x$component.vector))
 
-  writeLines(paste("component.diameters"))
-  print(t(x$diameters))
+  #writeLines(paste("component.diameters"))
+  #print(t(x$diameters))
   
   writeLines(paste("Symmetric:",x$symmetric))
 
